<?php
// $Id: og_ui.pages.inc,v 1.1.2.5 2010/12/15 09:14:12 amitaibu Exp $

/**
 * @file
 * Page callbacks for Group module.
 */

/**
 * Manage user subscription to a group.
 *
 * @param $node
 *   The group node entity.
 * @param $uid
 *   Optional; The user ID of the subscribing user. If empty the current user
 *   will be used.
 */
function og_ui_subscribe($gid, $uid = NULL) {
  global $user;
  if ($group = node_load($gid)) {
    if (empty($uid)) {
      $account = clone($user);

      if (!$account->uid) {
        $dest = drupal_get_destination();
        if (variable_get('user_register', 1)) {
          drupal_set_message(t('In order to join any group, you must <a href="!login">login</a>. After you have successfully done so, you will need to request membership again.', array('!login' => url("user/login", array('query' => $dest)))));
        }
        else {
          drupal_set_message(t('In order to join any group, you must <a href="!login">login</a> or <a href="!register">register</a> a new account. After you have successfully done so, you will need to request membership again.', array('!register' => url("user/register", array('query' => $dest)), '!login' => url("user/login", array('query' => $dest)))));
        }
        drupal_goto('user');
      }
    }
    else {
      $account = user_load($uid);
    }

    // Check user isn't already subscribed.
    if (in_array($group->nid, array_keys($account->og_groups))) {
      $params = array(
        '@user' => $account->name,
        // TODO: check user has permissions to see label (e.g. node title).
        '@group' => $group->title,
      );

      $message = $account->uid == $user->uid ? t('You are already a member of the group @group.', $params) : t('@user is already a member of the group @group.', $params);
      drupal_set_message($message);
      drupal_goto('node/' . $group->nid);
    }
    elseif (og_user_access($group->nid, 'subscribe', $account) || og_user_access($group->nid, 'subscribe without approval', $account)) {
      // Show the user a subscription confirmation.
      return drupal_get_form('og_ui_confirm_subscribe', $group, $account);
    }
    drupal_access_denied();
  }
  // Not a valid group node.
  drupal_not_found();
}

/**
 * Confirm subscribe form.
 */
function og_ui_confirm_subscribe($form_state, $group, $account) {
  $form['group'] = array('#type' => 'value', '#value' => $group);
  $form['account'] = array('#type' => 'value', '#value' => $account);
  if (!og_user_access($group->nid, 'subscribe without approval')) {
    $form['request'] = array(
      '#type' => 'textarea',
      '#title' => t('Additional details'),
      '#description' => t('Add any detail which will help an administrator decide whether to approve or deny your membership request.')
    );
  }
  $label = $group->title;
  return confirm_form($form, t('Are you sure you want to join the group %title?', array('%title' => $label)), "$group->entity_type/$group->etid", ' ', t('Join'), t('Cancel'));
}

/**
 * Submit handler; Confirm OG membership.
 */
function og_ui_confirm_subscribe_submit($form, &$form_state) {
  $request = !empty($form_state['values']['request']) ? $form_state['values']['request'] : '';

  $group = $form_state['values']['group'];
  $account = $form_state['values']['account'];
  $state = og_user_access($group->nid, 'subscribe without approval') ? OG_STATE_ACTIVE : OG_STATE_PENDING;

  // BACKPORT - Need to use the backport function here.
  _og_group($group->nid, $account->uid, $state);

  $form_state['redirect'] = "node/{$group->nid}";
}


/**
 * Confirm OG unsubscription form.
 *
 * The unsubscribing user is always the acting user - like this we make sure
 * no malicious user will unsubscribe another user. Administrators can reject or
 * ban another group member from the "people" page.
 */
function og_ui_unsubscribe($gid) {
  if ($group = node_load($gid)) {
    global $user;
    $label = $group->title;
    if (!empty($group->uid) && $group->uid != $user->uid) {
      if (in_array($group->nid, array_keys($user->og_groups))) {
        // Show the user a subscription confirmation.
        return drupal_get_form('og_ui_confirm_unsubscribe', $group, $user);
      }
    }
    else {
      drupal_set_message(t('As the manager of %group, you can not leave the group.', array('%group' => $label)));
    }
    drupal_goto('node/' . $group->nid);
  }
  // Not a valid group.
  drupal_not_found();
}

/**
 * Confirm unsubscribe form.
 */
function og_ui_confirm_unsubscribe($form_state, $group, $account) {
   $form['group'] = array('#type' => 'value', '#value' => $group);
   $form['account'] = array('#type' => 'value', '#value' => $account);
   $label = $group->title;
   return confirm_form($form, t('Are you sure you want to unsubscribe from the group %title?', array('%title' => $label)), "node/{$group->nid}", ' ', t('Remove'), t('Cancel'));
}

/**
 * Submit handler; Confirm OG unsubscription.
 */
function og_ui_confirm_unsubscribe_submit($form, &$form_state) {
  $group = $form_state['values']['group'];
  $account = $form_state['values']['account'];

  og_delete_subscription( $group->nid, $account->uid );

  // Determine where to go next - Group if accessible, or else site front page.
  $form_state['redirect'] = "node/{$group->nid}";
}